/*
 * ivinfo.h (JAN-2006)
 *
 * This file is a part of the Sophos Anti-Virus Interface (SAVI)(tm).
 *
 * Copyright (C) 2006 Sophos Plc, Oxford, England.
 * All rights reserved.
 *
 * This source code is only intended as a supplement to the
 * SAVI(tm) Reference and related documentation for the library.
 *
 * Sophos IQueryLoadedProtection and related declarations.
*/




#ifndef __IVINFO_H__
#define __IVINFO_H__

#include "isavi3.h"



/* Forward class declarations */
class IIdentityInfo;


typedef IEnum<IIdentityInfo *> IEnumIdentityInfo;  /* List of matching identity names.       */



class IQueryLoadedProtection : public SAVI_IUNKNOWN
{
public:
   virtual HRESULT SOPHOS_STDCALL GetMatchingIdentities( LPCOLESTR pIdentityName, REFIID IdListIID, void** ppIdList, U32* pMatchingIdCount ) = 0;
      /* Description:
            Check to see if the currently loaded engine and data combination detects a
            named identity. If a match is found the function also returns an enumeration
            of the matches encountered.

            pIdentityName   The substring used to match against identity names.

            IdListIID       Identifies the type of the returned object details. In this
                            case an enumeration of matching identies. This must be set
                            to SOPHOS_IID_ENUM_IDENTITY_INFO.

            ppIdList        A pointer to a pointer to an IEnumIdentityInfo object 
                            which will be created by this function. The object can be used
                            by the caller to enumerate any identities currently loaded
                            by the engine that match the name passed in.
                            If there are none, an enumerator containing zero items is
                            returned. When the caller has finished with the enumerator
                            object, they must call Release() on the object.

            pMatchingIdCount A pointer to a U32 to receive the no. of matching ID's

         Return value:            
            Success codes:
            S_OK           A match, or multiple matches, were found to the pIdentityName
                           string. The caller will have to determine from the returned
                           enumeration whether the match is what they were after.
                           If no match is encountered the enumeration will be empty.

            Failure codes:
            SOPHOS_SAVI_ERROR_NOT_INITIALISED
                           Neither initialisation function has been called.
            SOPHOS_SAVI_ERROR_UPGRADE_FAILED
                           An attempt to upgrade the Virus Engine failed and the SAVI
                           interface is temporarily unavailable.
            E_INVALIDARG   ppMatchingId was non-NULL but did not point to a valid
                           memory location.
            E_NOINTERFACE  IdentityInfoIID specified a configuration object type not
                           supported by this version of SAVI.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */

   virtual HRESULT SOPHOS_STDCALL GetAllIdentities( REFIID IdListIID, void** ppIdList, U32* pMatchingIdCount ) = 0;
      /* Description:
            Returns the complete enumeration of all identities loaded. 

            IdListIID       Identifies the type of the returned object details. In this
                            case an enumeration of matching identies. This must be set
                            to SOPHOS_IID_ENUM_IDENTITY_INFO.

            ppIdList        A pointer to a pointer to an IEnumIdentityInfo object 
                            which will be created by this function. 
                            The object can be used by the caller to enumerate through the 
                            virus identities currently loaded by the Virus Engine.
                            When the caller has finished with the enumerator object, it
                            must call Release() on the object.

            pMatchingIdCount A pointer to a U32 to receive the no of matching ID's

         Return value:
            Success codes:
            S_OK           The returned enumerator contains details of all the identities
                           that the currently loaded engine and data can detect.
                           If no identities are found the enumeration will be empty.

            Failure codes:
            SOPHOS_SAVI_ERROR_NOT_INITIALISED
                           Neither initialisation function has been called.
            SOPHOS_SAVI_ERROR_UPGRADE_FAILED
                           An attempt to upgrade the Virus Engine failed and the SAVI
                           interface is temporarily unavailable.
            E_INVALIDARG   ppMatchingId was non-NULL but did not point to a valid
                           memory location.
            E_NOINTERFACE  IdentityInfoIID specified a configuration object type not
                           supported by this version of SAVI.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */
   virtual HRESULT SOPHOS_STDCALL GetSingleIdentity( LPCOLESTR pIdentityName, REFIID IdentityIID, void** ppIdentity ) = 0;
      /* Description:
            Returns details of a single identity. 

            IdentityIID     Identifies the type of the returned object details. In this
                            case an identity info object. This must be set to
                            SOPHOS_IID_ENUM_IDENTITY_INFO .

            ppIdentity      A pointer to a pointer to an IIdentityInfo object which
                            will be created by this function.
                            When the caller has finished with the IIdentityInfo object,
                            it must call Release() on the object.

         Return value:
            Success codes:
            S_OK           The object contains details of the passed identity,
                           if one exists with exactly that name.
                           If no mathing identity is found the object will be NULL.

            Failure codes:
            SOPHOS_SAVI_ERROR_NOT_INITIALISED
                           Neither initialisation function has been called.
            SOPHOS_SAVI_ERROR_UPGRADE_FAILED
                           An attempt to upgrade the Virus Engine failed and the SAVI
                           interface is temporarily unavailable.
            E_INVALIDARG   ppMatchingId was non-NULL but did not point to a valid
                           memory location.
            E_NOINTERFACE  IdentityInfoIID specified a configuration object type not
                           supported by this version of SAVI.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */
};



class IIdentityInfo : public SAVI_IUNKNOWN
/*
   This class represents a single loaded virus identity found to match
   the requested input to IQueryLoadedProtection::GetSingleIdentity() or 
   one of the items in the enumerations returned by 
   IQueryLoadedProtection::GetMatchingIdentities or ...GetAllIdentities().
*/
{
public:
   virtual HRESULT SOPHOS_STDCALL GetName( U32 ArraySize, LPOLESTR pName, U32* pNameLength ) = 0;
      /* Description:
            Retrieve the full name of the identity.
   
         Parameters:
            ArraySize      The total size of the buffer supplied to receive the name, IN
                           CHARACTERS.
            pName          A pointer to the buffer which will receive the name.
                           NULL may be supplied if this value is not required.
            pNameLength    This location receives the total length of the matching identity
                           name INCLUDING TERMINATOR, in characters.

         Return value:            
            Success codes:
            S_OK           The buffer suppled was big enough and the name was copied into it.
                           
            Failure codes:
            SOPHOS_SAVI_ERROR_BUFFER_TOO_SMALL
                           The buffer supplied by the user was not big enough.
            E_INVALIDARG   A pointer parameter did not point to a valid memory location.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */

   virtual HRESULT SOPHOS_STDCALL GetNameWithoutType( U32 ArraySize, LPOLESTR pName, U32* pNameLength ) = 0;
      /* Description:
            Retrieve the name of the identity without any 'type' prefix but still including 
            any variant information. (e.g. if the full name is "W32/Blaster-C", then the
            function will return "Blaster-C")
   
         Parameters:
            ArraySize      The total size of the buffer supplied to receive the name, IN
                           CHARACTERS.
            pName          A pointer to the buffer which will receive the name.
                           NULL may be supplied if this value is not required.
            pNameLength    This location receives the total length of the matching identity
                           name INCLUDING TERMINATOR, in characters.

         Return value:            
            Success codes:
            S_OK           The buffer suppled was big enough and the name was copied into it.
                           
            Failure codes:
            SOPHOS_SAVI_ERROR_BUFFER_TOO_SMALL
                           The buffer supplied by the user was not big enough.
            E_INVALIDARG   A pointer parameter did not point to a valid memory location.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */

   virtual HRESULT SOPHOS_STDCALL IsVariant( U32* pVariant ) = 0;
      /* Description:
            Determine if the particular identity is one of a series of variants
            (i.e. is there an appended section <-variant> designation).

         Parameters:
            pVariant      This location is set to TRUE or FALSE by the function
                          TRUE means that this is one of a series of variants.
         Return value:            
            Success codes:
            S_OK          Function correctly returned data.

            Failure codes:
            E_INVALIDARG   A pointer parameter did not point to a valid memory location.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */

   virtual HRESULT SOPHOS_STDCALL IsFamily( U32* pFamily ) = 0;
      /* Description:
            Determine if the particular identity is a family identity

         Parameters:
            pFamily       This location is set to TRUE or FALSE by the function
                          TRUE means that this is a family identity.
         Return value:            
            Success codes:
            S_OK          Function correctly returned data.

            Failure codes:
            E_INVALIDARG   A pointer parameter did not point to a valid memory location.
            E_UNEXPECTED   An unexpected error.
            Win32 platforms may also return other HRESULT values.
      */

};


#endif   /*__IVINFO_H__ */
